<?php
/**
 * @file
 * Additional functions for HeadJS
 */

/**
 * Get the path of head.min.js file
 * @return string
 */
function headjs_get_path() {
  // decide which head.js to use; the loader-only or loader + tools & get file path
  $headjs_version = (variable_get('headjs_version', 'full') === 'full') ? HEADJS_FULL : HEADJS_LOADER;
  $path = headjs_path($headjs_version);
  return $path;
}

/**
 * Get the path of head.js library
 * Libraries API & CDN aware
 *
 * @param string
 * @return string
 */
function headjs_path($headjs_version) {
  $path = function_exists('libraries_get_path') ? libraries_get_path('headjs') : 'sites/all/libraries/headjs';

  if (file_exists("$path/dist/$headjs_version")) {
    $path .= "/dist/$headjs_version";
  } else if ($path) {
    $path .= "/$headjs_version";
  }

  // CDN Support.
  if (module_exists('cdn')) {
    $status = variable_get(CDN_STATUS_VARIABLE, CDN_DISABLED);
    if (($status == CDN_ENABLED || ($status == CDN_TESTING && user_access(CDN_PERM_ACCESS_TESTING))) && variable_get(CDN_THEME_LAYER_FALLBACK_VARIABLE, FALSE) == FALSE) {
      return file_create_url($path);
    }
  }

  return $path;
}

/**
 * Centralize warning message here, since it's called more than once from different places
 */
function _headjs_warning_message() {
  drupal_set_message(t('Please <a href="http://headjs.com/#download">download</a> and copy the head.js script to your @dir folder', array('@dir' => '/sites/all/libraries/headjs')), 'error', FALSE);
}

/**
 * Return the version of head.js that is installed.
 *
 * This can be used by other modules' hook_requirements() to ensure that the
 * proper version of head.js is installed.
 */
function headjs_get_version() {
  $version = 0;
  $pattern = '#	Version		([0-9\.a-z]+)#';
  $filename = headjs_get_path();

  // Return the version of head.js installed
  $headjs = file_get_contents($filename);
  if (preg_match($pattern, $headjs, $matches)) {
    $version = $matches[1];
  }

  return $version;
}

/**
 * Convert $variables['scripts'] to an array
 *
 * @param array $variables
 * @return array
 */
function _headjs_get_scripts($variables) {
  $scripts = array();
  $pattern = "!(<script type=\"text\/javascript\"(.*?)src=\"(.*?))(.*?)(\"(.*?)><\/script>)!";
  $defer = array();

  // first extract filenames
  if (preg_match_all($pattern, $variables['scripts'], $matches) > 0) {
    for ($i = 0, $n = count($matches[2]); $i < $n; ++$i) {
      // separate scripts with defer attribute
      if ($matches[2][$i] !== ' ' && substr_count($matches[2][$i], 'defer') >= 2) {
        $defer[] = $matches[4][$i];
      } else {
        $scripts[] = $matches[4][$i];
      }
    }
  }

  // add deferred scripts to the end of the queue
  $scripts = array_merge($scripts, $defer);

  // now extract settings
  $cdata = '<!--//--><![CDATA[//><!--';
  $pos = strpos($variables['scripts'], $cdata);
  $temp = substr($variables['scripts'], $pos);
  $replace = array('<script type="text/javascript">', '</script>', $cdata, '//--><!]]>');
  $settings = (strpos($temp, $cdata) !== FALSE) ? trim(str_replace($replace, '', $temp)) : NULL;

  return array($scripts, $settings);
}

/**
 * Replace inline scripts with calls to head.js()
 *
 * @param string $content
 * @param array $headscripts
 * @param array $inline
 * @param string $key
 */
function _headjs_replace(&$content, &$headscripts, &$inline, $key = NULL) {
  $matches = array();
  preg_match('#<script .+?</script>#s', $content, $matches);

  foreach ($matches as $match) {
    $matches_scriptname = array();
    if (preg_match('#src="(.+?)"#', $match, $matches_scriptname)) {

      // elements in <head> go the beginning of the line
      if ($key === 'head') {
        array_unshift($headscripts, $matches_scriptname[1]);
      } else {
        $headscripts[] = $matches_scriptname[1];
      }

      $content = str_replace($match, '', $content);
    } else {
      $inline[] = trim(str_replace(array('<script type="text/javascript">', '<!--//--><![CDATA[//><!--', '</script>', '//--><!]]>'), '', $match));
      $content = str_replace($match, '', $content);
    }
  }
}

/**
 * Rewrites <script> tag to use HeadJS
 * Borrowed from LABjs
 *
 * @param $javascript
 *   String including <script> tag
 * @return
 *   Input string is modified and returned
 */
function headjs_rewrite_js(&$javascript) {
  $javascript = preg_replace_callback('#<script .+?</script>\s*#s', '_headjs_rewrite_js', $javascript);
  return $javascript;
}

/**
 * Callback function for headjs_rewrite_js().
 * Borrowed from LABjs
 *
 * When called without parameter, returns an array containing all processed scripts.
 */
function _headjs_rewrite_js($matches = NULL) {
  // We store all processed scripts in the $scripts array.
  static $scripts = array();

  if (!$matches) {
    return $scripts;
  } else {
    // If this is an inline script, we remove and add the src to $scripts array.
    // If not, we try to wrap it with head.ready()
    if (preg_match('#type="text/javascript" (defer="defer" |)src="(.+?)"#', $matches[0], $match)) {
      if (strpos($matches[0], 'tinymce/jscripts/tiny_mce/tiny_mce.js') == FALSE) {
        $scripts[] = "\"{$match[2]}\"";
        return '';
      } else {
        return $matches[0];
      }
    } else {
      return $matches[0];
    }
  }
}

/*
 * Integration with "Advanced CSS/JS Aggregation"
 * http://drupal.org/project/advagg
 */

/**
 * Build and theme JS output.
 *
 * @param $external_no_preprocess
 *   array(array($src, $defer))
 * @param $output_preprocess
 *   array(array($src, $prefix, $suffix))
 * @param $output_no_preprocess
 *   array(array(array($src, $defer)))
 * @param $setting_no_preprocess
 *   array(array($code))
 * @param $inline_no_preprocess
 *   array(array($code, $defer))
 * @param $scope
 *   header or footer
 * @return
 *   String of themed JavaScript.
 */
function headjs_advagg_js_builder($external_no_preprocess, $output_preprocess, $output_no_preprocess, $setting_no_preprocess, $inline_no_preprocess, $scope = 'header') {
  // For inline Javascript to validate as XHTML, all Javascript containing
  // XHTML needs to be wrapped in CDATA. To make that backwards compatible
  // with HTML 4, we need to comment out the CDATA-tag.
  $embed_prefix = "\n<!--//--><![CDATA[//><!--\nhead.ready(function() {";
  $embed_suffix = "});\n//--><!]]>\n";

  // Keep the order of JS files consistent as some are preprocessed and others are not.
  // Make sure any inline or JS setting variables appear last after libraries have loaded.
  $scripts = array();
  $excluded = array();

//  krumo($external_no_preprocess, $output_preprocess, $output_no_preprocess, $setting_no_preprocess, $inline_no_preprocess);die;

  foreach ($external_no_preprocess as $values) {
    list ($src, $defer) = $values;
    $scripts[] = $src;
  }

  foreach ($output_preprocess as $values) {
    list ($src, $prefix, $suffix) = $values;
    $scripts[] = $src;
  }

  foreach ($output_no_preprocess as $type => $list) {
    if (!empty($list)) {
      foreach ($list as $values) {
        list ($src, $defer) = $values;
        if (strpos($src, 'tinymce/jscripts/tiny_mce/tiny_mce.js') === FALSE) {
          $scripts[] = $src;
        } else {
          $excluded[] = $values;
        }
      }
    }
  }

//  krumo($scripts, $excluded, $scope);

  if ($scope == 'header') {
    $headjs_path = headjs_get_path();

    // Add HeadJS script
    if (variable_get('headjs_embed', FALSE)) {
      // as inline script
      $output = '<script type="text/javascript">' . "\n";
      $output .= '<!--//--><![CDATA[//><!--' . "\n";
      $output .= file_get_contents($headjs_path);
      $output .= "//--><!]]>\n";
      $output .= "</script>\n";
    } else {
      // as external script
      if ($headjs_path[0] !== '/' && strpos($headjs_path, 'http://') !== 0) {
        $headjs_path = '/' . $headjs_path;
      }
      $output = '<script type="text/javascript" src="' . $headjs_path . '"></script>' . "\n";
    }

    if (!empty($scripts)) {
      $output .= '<script type="text/javascript">' . "\n";
      $output .= '<!--//--><![CDATA[//><!--' . "\n";
      $output .= "head.js(\"" . implode("\");\nhead.js(\"", $scripts) . "\"); \n";
      $output .= "//--><!]]>\n";
      $output .= "</script>\n";
    }
  } elseif (!empty($scripts)) {
    $output .= '<script type="text/javascript">' . "\n";
    $output .= '<!--//--><![CDATA[//><!--' . "\n";
    $output .= "head.js(\"" . implode("\");\nhead.js(\"", $scripts) . "\"); \n";
    $output .= "//--><!]]>\n";
    $output .= "</script>\n";
  }

  foreach ($excluded as $values) {
    list ($src, $defer) = $values;
    $output .= '<script type="text/javascript"' . ($defer ? ' defer="defer"' : '') . ' src="' . $src . "\"></script>\n";
  }

  foreach ($setting_no_preprocess as $code) {
    $output .= '<script type="text/javascript">' . $embed_prefix . $code . $embed_suffix . "</script>\n";
  }

  foreach ($inline_no_preprocess as $values) {
    list ($code, $defer) = $values;
    $output .= '<script type="text/javascript"'. ($defer ? ' defer="defer"' : '') .'>' . $embed_prefix . $code . $embed_suffix . "</script>\n";
  }

//  krumo($output);die;

  return $output;
}

/*
 * Integration with "Do"
 * http://drupal.org/project/do
 */

/**
 * Enable HeadJS via Do
 */
function headjs_on() {
  variable_set('headjs_enable', TRUE);
  drupal_set_message(t('HeadJS enabled!'));
  return array(
    'ok' => TRUE,
    'redirect' => TRUE,
    'msg' => t('HeadJS enabled!')
  );
}

/**
 * Disable HeadJS via Do
 */
function headjs_off() {
  variable_set('headjs_enable', FALSE);
  drupal_set_message(t('HeadJS disabled!'));
  return array(
    'ok' => TRUE,
    'redirect' => TRUE,
    'msg' => t('HeadJS disabled!')
  );
}